@@ -4,7 +4,7 @@ use 5.008;
 use strict;
 use warnings;
 
-use AutoLoader qw(AUTOLOAD);
+#use AutoLoader qw(AUTOLOAD);
 
 use Encode::Encoding;
 use Pod::LaTeX;
@@ -13,13 +13,13 @@ use Carp;
 
 our @ISA = qw(Encode::Encoding);
 
-our $VERSION = '0.8';
+our $VERSION = '0.9';
 
 use constant ENCODE_CHRS => '<>&"';
 
 __PACKAGE__->Define(qw(LaTeX BibTeX latex bibtex));
 
-use vars qw( %LATEX_Escapes %LATEX_Escapes_inv %LATEX_Math_mode $LATEX_Math_mode_re $LATEX_Reserved );
+use vars qw( %LATEX_Escapes %LATEX_Escapes_inv $LATEX_Escapes_inv_re %LATEX_Math_mode $LATEX_Math_mode_re $LATEX_Reserved );
 
 # Missing entities in HTML::Entities?
 @HTML::Entities::entity2char{qw(sol verbar)} = qw(\textfractionsolidus{} |);
@@ -35,12 +35,13 @@ while( my ($entity,$tex) = each %Pod::LaTeX::HTML_Escapes ) {
 	# 1.27 used UTF-8 in the source, which requires decoding
 	utf8::decode($c) if $HTML::Entities::VERSION < 1.35;
 	
+	$tex =~ s/\{\}$//; # Strip the trailing {}
 	$LATEX_Escapes{$c} = $tex;
 	if( $tex =~ s/^\$\\(.+)\$/$1/ ) {
 		$LATEX_Math_mode{$tex} = $c;
 #		warn "MM: ", quotemeta($tex), " => ", $c, "\n";
 	} elsif( $tex =~ s/^\\// ) {
-		$LATEX_Escapes_inv{quotemeta($tex)} = $c;
+		$LATEX_Escapes_inv{$tex} = $c;
 #		warn quotemeta($tex), " => ", $c, "\n";
 	}
 }
@@ -62,7 +63,36 @@ while( my ($entity,$tex) = each %Pod::LaTeX::HTML_Escapes ) {
 	$LATEX_Escapes{
 		$LATEX_Escapes_inv{'Lambda'} = $LATEX_Escapes_inv{'Lamda'}
 	} = "\\Lambda";
+	# Special Characters from
+	# http://www.cs.wm.edu/~mliskov/texsymbols.pdf
+	my %euro = (
+		chr(0x2020) => 'dag', # Dagger
+		chr(0x2021) => 'ddag', # Double-dagger
+		chr(0xa7) => 'S', # Section mark
+		chr(0xb6) => 'P', # Paragraph
+		chr(0xdf) => 'ss', # German sharp S
+		chr(0x152) => 'OE', # French ligature OE
+		chr(0x153) => 'oe', # French ligature oe
+		chr(0x141) => 'L', # Polish suppressed-l
+		chr(0x142) => 'l', # Polish suppressed-L
+		chr(0xd8) => 'O', # Scandinavian O-with-slash
+		chr(0xf8) => 'o', # Scandinavian o-with-slash
+		chr(0xc5) => 'AA', # Scandinavian A-with-circle
+		chr(0xe5) => 'aa', # Scandinavian a-with-circle
+		chr(0x131) => 'i', # dotless i
+		chr(0x237) => 'j', # dotless j
+	);
+	while(my($c,$tex) = each %euro)
+	{
+		$LATEX_Escapes{$LATEX_Escapes_inv{$tex} = $c} = "\\$tex";
+	}
 }
+# Build a single regexp for LaTeX macros
+$LATEX_Escapes_inv_re =
+	join '|',
+	map { quotemeta($_) }
+	sort { length($b) <=> length($a) }
+	keys %LATEX_Escapes_inv;
 
 # Math-mode sequences
 %LATEX_Math_mode = (
@@ -86,10 +116,12 @@ while( my ($entity,$tex) = each %Pod::LaTeX::HTML_Escapes ) {
 	'R' => chr(0x211d),
 	'Z' => chr(0x2124),
 );
-foreach my $re (sort { length($b) <=> length($a) } keys %LATEX_Math_mode) {
-	$LATEX_Math_mode_re .= quotemeta($re).'|';
-}
-chop($LATEX_Math_mode_re);
+# Build a single regexp for math mode macros
+$LATEX_Math_mode_re =
+	join '|',
+	map { quotemeta($_) }
+	sort { length($b) <=> length($a) }
+	keys %LATEX_Math_mode;
 # TODO
 # e.g. \acute{e} => \'e
 # Math-mode accents: hat, acute, bar, dot, breve, check, grave, vec, ddot, tilde
@@ -104,24 +136,27 @@ sub encode
 	my ($self,$str,$check) = @_;
 	$str =~ s/([$LATEX_Reserved])/\\$1/sog;
 	$str =~ s/([<>])/\$$1\$/sog;
-	$str =~ s/([^\x00-\x80])/$LATEX_Escapes{$1}/sg;
+	$str =~ s/([^\x00-\x80])(?![A-Za-z0-9])/$LATEX_Escapes{$1}/sg;
+	$str =~ s/([^\x00-\x80])/$LATEX_Escapes{$1}\{\}/sg;
 	return $str;
 }
+
 # decode($octets [,$check])
 sub decode
 {
 	my ($self,$str,$check) = @_;
-	# Convert standard macros to chars
-	while( my ($re,$char) = each %LATEX_Escapes_inv ) {
-		$str =~ s/\\$re/$char/sg;
-	}
-	# Additionally convert mathmode macros to unicode
+
+	# Convert mathmode macros to unicode
 	$str =~ s/\$([^\$]+?)\$/'$'.&_mathmode($1).'$'/seg;
+
+	# Convert standard macros to chars
+	$str =~ s/\\($LATEX_Escapes_inv_re)/$LATEX_Escapes_inv{$1}/sg;
 	
 	# $str = encode_entities($str,'<>&"');
 	# Convert some LaTeX macros into HTML equivalents
 	return _htmlise(\$str);
 }
+
 # Math-mode symbols
 sub _mathmode
 {
@@ -255,7 +290,7 @@ Timothy D Brody, E<lt>tdb01r@ecs.soton.ac.ukE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2006 by Timothy D Brody
+Copyright (C) 2005-2007 by Timothy D Brody
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.7 or,
